@@ -21,8 +21,8 @@
  *
  * CVS Information:
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager.h,v $
- * $Date: 2011/12/08 20:50:31 $
- * $Revision: 1.70.2.1 $
+ * $Date: 2012/03/27 15:19:02 $
+ * $Revision: 1.70.2.5 $
  * $Author: bester $
  */
 #ifndef GLOBUS_GRAM_JOB_MANAGER_INCLUDE
@@ -41,6 +41,8 @@
 
 EXTERN_C_BEGIN
 
+#define GLOBUS_GRAM_JOB_MANAGER_EXPIRATION_ATTR "expiration"
+
 /** Pointer to the current request to allow per-job logging to occur */
 extern globus_thread_key_t globus_i_gram_request_key;
 
@@ -84,11 +86,11 @@ typedef enum
     GLOBUS_GRAM_JOB_MANAGER_STATE_STOP,
     GLOBUS_GRAM_JOB_MANAGER_STATE_POLL_QUERY1,
     GLOBUS_GRAM_JOB_MANAGER_STATE_POLL_QUERY2,
-    GLOBUS_GRAM_JOB_MANAGER_STATE_PROXY_REFRESH,
-    GLOBUS_GRAM_JOB_MANAGER_STATE_PRE_CLOSE_OUTPUT,
+    /* GLOBUS_GRAM_JOB_MANAGER_STATE_PROXY_REFRESH  OBSOLETE STATE, */
+    GLOBUS_GRAM_JOB_MANAGER_STATE_PRE_CLOSE_OUTPUT = GLOBUS_GRAM_JOB_MANAGER_STATE_POLL_QUERY2+2,
     GLOBUS_GRAM_JOB_MANAGER_STATE_TWO_PHASE_QUERY1,
-    GLOBUS_GRAM_JOB_MANAGER_STATE_TWO_PHASE_QUERY2,
-    GLOBUS_GRAM_JOB_MANAGER_STATE_TWO_PHASE_PROXY_REFRESH
+    GLOBUS_GRAM_JOB_MANAGER_STATE_TWO_PHASE_QUERY2
+    /* GLOBUS_GRAM_JOB_MANAGER_STATE_TWO_PHASE_PROXY_REFRESH  OBSOLETE STATE */
 }
 globus_gram_jobmanager_state_t;
 
@@ -434,8 +436,12 @@ typedef struct globus_gram_job_manager_s
      * Callback handle for fork SEG-like polling
      */
     globus_callback_handle_t            fork_callback_handle;
-    /** Scheduler-specific set of validation records */
+    /** LRM-specific set of validation records */
     globus_list_t *                     validation_records;
+    /** Newest validation file timestamp */
+    time_t                              validation_record_timestamp;
+    /** Track when validation files are added or removed */
+    globus_bool_t                       validation_file_exists[4];
     /** GRAM job manager listener contact string */
     char *                              url_base;
     /** Time when the job manager-wide proxy will expire */
@@ -498,6 +504,18 @@ typedef struct globus_gram_job_manager_s
      */
     char *                              gt3_failure_message;
     globus_xio_attr_t                   script_attr;
+
+    /**
+     * Periodic callback handle to expire jobs which completed or failed
+     * but didn't have two-phase end happen.
+     */
+    globus_callback_handle_t            expiration_handle;
+
+    /**
+     * Periodic callback handle to abort if something removes the lock file.
+     */
+    globus_callback_handle_t            lockcheck_handle;
+    
 }
 globus_gram_job_manager_t;
 
@@ -663,8 +681,6 @@ typedef struct globus_gram_job_manager_request_s
     char *                              job_contact_path;
     /** Job-specific persistence file */
     char *                              job_state_file;
-    /** Job-specific persistence lock file */
-    char *                              job_state_lock_file;
     /** Job-specific scratch directory after RSL evaluation */
     char *                              scratch_dir_base;
     /**
@@ -680,8 +696,6 @@ typedef struct globus_gram_job_manager_request_s
     char *                              remote_io_url_file;
     /** Job-specific proxy file */
     char *                              x509_user_proxy;
-    /** Job-specific persistence lock descriptor */
-    int                                 job_state_lock_fd;
     /** Thread safety */
     globus_mutex_t                      mutex;
     /** Thread safety */
@@ -803,6 +817,14 @@ typedef struct globus_gram_job_manager_ref_s
      */
     globus_bool_t                       loaded_only;
 
+    /**
+     * Timestamp of when to auto-destroy this job. Thsi will be 0 unless
+     * the job has completed and failed to have it's two-phase commit end.
+     * A periodic event will poll through the refs that have this attribute
+     * set, and will reload them with a fake commit to get them cleaned up.
+     */
+    time_t                              expiration_time;
+
     /* The following are used for the internal fakeseg stuff for condor*/
 
     /**
@@ -982,11 +1004,6 @@ typedef enum
 }
 globus_gram_job_manager_validation_when_t;
 
-extern
-int
-globus_gram_job_manager_validation_init(
-    globus_gram_job_manager_t *         config);
-
 extern
 int
 globus_gram_job_manager_validation_destroy(
@@ -1149,13 +1166,6 @@ globus_gram_job_manager_query_callback(
     int                                 errorcode,
     char *                              uri);
 
-void
-globus_gram_job_manager_query_delegation_callback(
-    void *                              arg,
-    globus_gram_protocol_handle_t       handle,
-    gss_cred_id_t                       credential,
-    int                                 error_code);
-
 void
 globus_gram_job_manager_query_reply(
     globus_gram_jobmanager_request_t *  request,
@@ -1301,8 +1311,7 @@ globus_gram_rsl_add_stream_out(
 int
 globus_gram_job_manager_state_file_set(
     globus_gram_jobmanager_request_t *  request,
-    char **                             state_file,
-    char **                             state_lock_file);
+    char **                             state_file);
 
 int
 globus_gram_job_manager_file_lock(
@@ -1507,6 +1516,10 @@ void
 globus_gram_job_manager_set_grace_period_timer(
     globus_gram_job_manager_t *         manager);
 
+void
+globus_gram_job_manager_expire_old_jobs(
+    void *                              arg);
+
 int
 globus_gram_job_manager_set_status(
     globus_gram_job_manager_t *         manager,
