@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_config.c,v $
- * $Date: 2011/10/20 18:11:48 $
- * $Revision: 1.24 $
+ * $Date: 2012/04/11 06:00:29 $
+ * $Revision: 1.24.4.1 $
  * $Author: bester $
  */
 
@@ -614,7 +614,7 @@ globus_gram_job_manager_config_init(
 
     rc = uname(&utsname);
 
-    if (rc == 0)
+    if (rc >= 0)
     {
         if (config->globus_host_osname == NULL)
         {
