@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/main.c,v $
- * $Date: 2011/11/22 17:55:33 $
- * $Revision: 1.41 $
+ * $Date: 2012/03/14 18:42:28 $
+ * $Revision: 1.41.2.3 $
  * $Author: bester $
  */
 
@@ -71,6 +71,10 @@ void
 globus_l_gram_cputype_and_manufacturer(
     globus_gram_job_manager_config_t *  config);
 
+static
+void
+globus_l_gram_lockcheck(
+    void *                              arg);
 #endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
 
 int
@@ -91,6 +95,7 @@ main(
     OM_uint32                           major_status, minor_status;
     pid_t                               forked_starter = 0;
     globus_bool_t                       cgi_invoked = GLOBUS_FALSE;
+    int                                 lock_tries_left = 10;
 
     if ((sleeptime_str = getenv("GLOBUS_JOB_MANAGER_SLEEP")))
     {
@@ -395,6 +400,11 @@ main(
         else if (rc != GLOBUS_GRAM_PROTOCOL_ERROR_OLD_JM_ALIVE)
         {
             /* Some system error. Try again */
+            if (--lock_tries_left == 0)
+            {
+                reply_and_exit(NULL, rc, "Unable to create lock file");
+            }
+            sleep(1);
             continue;
         }
 
@@ -482,6 +492,32 @@ main(
                         &manager);
                         
             }
+
+            {
+                globus_reltime_t        expire_period;
+
+                GlobusTimeReltimeSet(expire_period, 1, 0);
+
+                rc = globus_callback_register_periodic(
+                    &manager.expiration_handle,
+                    NULL,
+                    &expire_period,
+                    globus_gram_job_manager_expire_old_jobs,
+                    &manager);
+            }
+
+            {
+                globus_reltime_t        lockcheck_period;
+
+                GlobusTimeReltimeSet(lockcheck_period, 60, 0);
+
+                rc = globus_callback_register_periodic(
+                    &manager.lockcheck_handle,
+                    NULL,
+                    &lockcheck_period,
+                    globus_l_gram_lockcheck,
+                    &manager);
+            }
         }
         else if (http_body_fd >= 0)
         {
@@ -584,6 +620,14 @@ main(
     {
         GlobusGramJobManagerWait(&manager);
     }
+    if (manager.expiration_handle != GLOBUS_NULL_HANDLE)
+    {
+        globus_callback_unregister(manager.expiration_handle, NULL, NULL, NULL);
+    }
+    if (manager.lockcheck_handle != GLOBUS_NULL_HANDLE)
+    {
+        globus_callback_unregister(manager.lockcheck_handle, NULL, NULL, NULL);
+    }
     GlobusGramJobManagerUnlock(&manager);
 
     globus_gram_job_manager_log(
@@ -942,4 +986,84 @@ globus_l_gram_cputype_and_manufacturer(
     }
 }
 
+static
+void
+globus_l_gram_lockcheck(
+    void *                              arg)
+{
+    globus_gram_job_manager_t *         manager = arg;
+    struct stat                         lockfile_stat = {0}, lockfd_stat = {0};
+    int                                 lockfile_errno = 0, lockfd_errno = 0;
+    int                                 rc;
+    char                               *msg1 = NULL, *msg2 = NULL, *msg3 = NULL;
+
+    errno = 0;
+
+    rc = stat(manager->lock_path, &lockfile_stat);
+    if (rc < 0)
+    {
+        msg1 = "Cannot stat lockfile";
+        lockfile_errno = errno;
+    }
+    rc = fstat(manager->lock_fd, &lockfd_stat);
+    if (rc < 0)
+    {
+        lockfd_errno = errno;
+        msg2 = "Cannot stat lockfd";
+    }
+
+    if (msg1 == NULL && msg2 == NULL)
+    {
+        if (lockfd_stat.st_ino != lockfile_stat.st_ino)
+        {
+            msg3 = "Lockfile replaced";
+        }
+    }
+
+    if (msg1 != NULL || msg2 != NULL || msg3 != NULL)
+    {
+        goto fatal;
+    }
+
+    return;
+
+fatal:
+    globus_gram_job_manager_log(
+            manager,
+            GLOBUS_GRAM_JOB_MANAGER_LOG_FATAL,
+            "event=gram.jobmanager.end "
+            "level=FATAL "
+            "message=\"Lockfile sanity check failed, aborting\" "
+            "lockfile=\"%s\" "
+            "lockfile_inode=%ld "
+            "lockfile_errno=%d "
+            "lockfile_message=\"%s\" "
+            "lockfd=%d "
+            "lockfd_inode=%ld "
+            "lockfd_errno=%d "
+            "lockfd_message=\"%s\" "
+            "%s%s%s"
+            "%s%s%s"
+            "%s%s%s"
+            "\n",
+            manager->lock_path,
+            (long int) lockfile_stat.st_ino,
+            lockfile_errno,
+            strerror(lockfile_errno),
+            manager->lock_fd,
+            (long int) lockfd_stat.st_ino,
+            lockfd_errno,
+            strerror(lockfile_errno),
+            (msg1 != NULL) ? "msg1=\"" : "",
+            (msg1 != NULL) ? msg1 : "",
+            (msg1 != NULL) ? "\" " : "",
+            (msg2 != NULL) ? "msg2=\"" : "",
+            (msg2 != NULL) ? msg2 : "",
+            (msg2 != NULL) ? "\" " : "",
+            (msg3 != NULL) ? "msg3=\"" : "",
+            (msg3 != NULL) ? msg3 : "",
+            (msg3 != NULL) ? "\" " : "");
+    abort();
+}
+/* globus_l_gram_lockcheck() */
 #endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */
